<?php
namespace Folha\Logic\ESocial\Eventos\Data;
use Folha\Repositories\GpeEsocialRegistro;

/**
 * Classe provedora do datasource utilizado para a geracao do evento S-1210 - Pagamentos de Rendimentos do Trabalho.
 *
 * @package Folha\Logic\ESocial\Eventos\Data
 * @author felipesantos
 **/
class S1210
{

    /**
     * Retorna os dados que serao utilizado para a geracao do(s) arquivo(s) do evento S-1210, Pagamentos de Rendimentos do Trabalho.
     *
     * @param $dsordem                 Define as possíveis formas de filtro utilizadas pela tela de envio de eventos  
     *                                 do e-Social. Pode ser ALFABETICA, MATRICULA, DATA_ADMISSAO e ESTRUTURA. Para este
     *                                 evento, em especial, somente o filtro por ESTRUTURA poderá será relevante.
     * @param $limiteInferior          Limite inferior do filtro aplicado. O significado dos dados trazidos neste campo
     *                                 deverao variar, conforme o parametro dsordem.
     * @param $limiteSuperior          Limite superior do filtro aplicado. O significado dos dados trazidos neste campo
     *                                 deverao variar, conforme o parametro dsordem.
     * @param $inicioPeriodo           Inicio do intervalo que deverá ter os eventos gerados. Somente será utilizado se
     *                                 o parametro "$utilizaIntervaloDeDatas" for igual a "true". 
     * @param $fimPeriodo              Final do intervalo que deverá ter os eventos gerados. Somente será utilizado se
     *                                 o parametro "$utilizaIntervaloDeDatas" for igual a "true".
     * @param $utilizaIntervaloDeDatas Verifica se o dataset deve ser filtrado pelos campos de intervalo da tela de  
     *                                 envio de eventos e-Social, ou se devem enviados os arquivos alterados depois do 
     *                                 ultimo envio.
     * @param $idRetificacao           Identificador de Retificação dos arquivos ou não.
     * @param $nrRecibo                Número do Recibo do primeiro envio em caso de Retificação.
     * @param $idApuracao              Identificador da apuração Mensal ou Anual.
     * @param $nrorg                   Nrorg.
     * @param $versaoHRM               Versao do sistema HRM utilizado para a geracao dos relatorios.
     *
     * @return array
     * @author felipesantos
     **/
    public static function getDataSource($dsordem, $limiteInferior, $limiteSuperior, $inicioPeriodo, $fimPeriodo, 
                                         $utilizaIntervaloDeDatas, $nrorg, $versaoHRM, $versaoEsocial, $tpAmbiente, $indRetificacao, $nrRecibo, $indApuracao)
    {
        $S1210 = \Folha\Repositories\GpeEsocialRegistro::dataSetEsocial_S1210(
            $dsordem, $limiteInferior, $limiteSuperior, $inicioPeriodo, $fimPeriodo, 
            $utilizaIntervaloDeDatas, $nrorg, $versaoHRM, $versaoEsocial, $tpAmbiente, $indRetificacao, $nrRecibo, $indApuracao
        );

        foreach ($S1210 as &$row) {

            $nrReciboResc = null; //@todo Verificar forma de buscar o número do recibo de rescisão do vínculo

            $infoPgto = null;
            $detPgtoFl = null;
            $detPgtoResc = null;

            $pagtos = GpeEsocialRegistro::s1210_infoPgto($inicioPeriodo, $fimPeriodo, $row['NRVINCULOM'], $nrorg);

            foreach ($pagtos as $pagto) {
                /*Detalhamento dos pagamentos relativos a folha de pagamento*/
                $pagtosFolha = GpeEsocialRegistro::s1210_detPgtoFl($inicioPeriodo, $fimPeriodo, $row['NRVINCULOM'], $pagto['DTOCORRENCIA'], $nrorg);
                foreach ($pagtosFolha as $pagtoFl) {
                    $detPgtoFl[] =
                        array(
                            "_EVTPGTOS_IDEBENEF_INFOPGTO_DETPGTOFL_PERREF" => $pagtoFl['PERREF'],
                            "_EVTPGTOS_IDEBENEF_INFOPGTO_DETPGTOFL_VLRPGTO" => $pagtoFl['VLRPGTO'],
                            "_EVTPGTOS_IDEBENEF_INFOPGTO_DETPGTOFL_IDERECPGTO" => $pagtoFl['IDERECPGTO']
                        );
                }

                /*Detalhamento dos pagamentos relativos a recisoes contratuais*/
                $pagtosResc = GpeEsocialRegistro::s1210_detPgtoResc($inicioPeriodo, $fimPeriodo, $row['NRVINCULOM'], $pagto['DTOCORRENCIA'], $nrReciboResc, $nrorg);
                foreach ($pagtosResc as $pagtoResc) {
                    $detPgtoFl[] =
                        array(
                            "_EVTPGTOS_IDEBENEF_INFOPGTO_DETPGTORESC_VLRPGTO" => $pagtoResc['VLRPGTO'],
                            "_EVTPGTOS_IDEBENEF_INFOPGTO_DETPGTORESC_IDERECPGTO" => $pagtoResc['IDERECPGTO'],
                            "_EVTPGTOS_IDEBENEF_INFOPGTO_DETPGTORESC_NRRECARQ" => $pagtoResc['NRRECARQ']
                        );
                }

                $infoPgto[] =
                    array(
                        "_EVTPGTOS_IDEBENEF_INFOPGTO_DTPGTO" => $pagto['DTPGTO'],
                        "_EVTPGTOS_IDEBENEF_INFOPGTO_INDRESBR" => $pagto['INDRESBR'],
                        "_EVTPGTOS_IDEBENEF_INFOPGTO_TPPGTO" => $pagto['TPPGTO'],
                        "_EVTPGTOS_IDEBENEF_INFOPGTO_DETPGTOFL" => $detPgtoFl,
                        "_EVTPGTOS_IDEBENEF_INFOPGTO_DETPGTORESC" => $detPgtoResc
                    );
            }

            $row['INFOPGTO'] = $infoPgto;
        }

        return $S1210;
    }

}