<?php

require_once 'base/FpaitecalcfolhaEntityBase.php';


/**
 * Enter description here ...
 *
 * @package Entity
 *
 * @author  alessandrobartels
 *
 */
class FpaitecalcfolhaEntity extends FpaitecalcfolhaEntityBase
{


    const VALOR = 'VAL';  /* Valor do item calculo */
    const REFERENCIA = 'REF'; /* Referencia do item calculo */
    const PARCELA = 'PRC'; /* Quantidade de ocorrencia do item calculo (parcelas) */
    const NENHUMA = 'N'; /* Indica nenhuma operacao a ser feita */
    const MAIS = '+'; /* Indica somatorio de valor */
    const MENOS = '-'; /* Indica diferenca de valor */

    const EVENTO_COMISSAO = '140';
    const EVENTO_SALARIO_MES = '10';
    const EVENTO_SALARIO_HORA = '20';

    private $_totalReferencias;
    private $evento;

    /**
     * Retorna o valor do evento (usado em regra).
     *
     * @param integer $indice Indice.
     *
     * @return float
     *
     * @author  alessandrobartels
     */
    public function retornaValor($indice){
        if($indice == -1){//leandroafonso 18/11/2013
            return 0;
        }
        return $this->getItem($indice)->getVreventofolha();
    }


    /**
     * Retorna o contador do evento (usado em regra).
     *
     * @param integer $indice Indice.
     *
     * @return integer
     *
     * @author alessandrobartels 30/08/2011
     */
    public function retornaContador($indice){
        return $this->getItem($indice)->getQtocorproxperi();
    }


    /**
     * Retorna numero do evento (usado em regra).
     *
     * @param integer $indice Indice.
     *
     * @return integer
     *
     * @author  alessandrobartels
     */
    public function retornaEvento($indice){
        return $this->getItem($indice)->getNreventom();
    }


    /**
     * Retorna numero da referencia (usado em regra).
     *
     * @param integer $indice Indice.
     *
     * @return integer
     *
     * @author  alessandrobartels
     */
    public function retornaReferencia($indice){
        return $this->getItem($indice)->getVrreffolha();
    }


    /**
     * Retorna Data de ocorrencia (usado em regra).
     *
     * @param integer $indice Indice.
     *
     * @return string
     *
     * @author  alessandrobartels
     */
    public function retornaDtocorrencia($indice){
        return $this->getItem($indice)->getDtocorrencia();
    }
    /**
     * Retorna nrvinculom (usado em regra).
     *
     * @param integer $indice Indice.
     *
     * @return string
     *
     * @author  leandroafonso
     */
    public function retornaNrvinculom($indice){
        return $this->getItem($indice)->getNrvinculom();
    }


    public function getTotalReferencias(){
        return $this->_totalReferencias;
    }


    /**
     * Atualiza atributo de evento demonstrativo.
     *
     * @param integer $posicao Posicao do evento na lista.
     * @param integer $id_demo ID Demo.
     *
     * @return void
     *
     * @author  alessandrobartels
     */
    public function atribuiCalculoDemonstrativo($posicao, $id_demo){
        // -1 indica que evento nao esta na lista.
        if ($posicao !== -1) {
            $item = $this->getItem($posicao);
            $item->setIdevendemoncalc($id_demo);
        }
    }


    /**
     * Verifica se o item calculo podera ser removido da lista caso o valor seje 0 e nao gerar valor zero.
     *
     * @param float                 $valor           Valor.
     * @param string                $idGeraValorZero Flag Gera valor zero.
     * @param FpaitecalcfolhaEntity $item            Instancia de FpaitecalcfolhaEntity.
     *
     * @return boolean
     */
    private function _verificaRemoveItem($valor, $idGeraValorZero, FpaitecalcfolhaEntity $item){
        /* A comparacao de uma variavel (float) com '0' deve ser feita com casa decimal (0.0). */
        if (($valor === 0.0) && ($idGeraValorZero === FpaeventohEntity::IDGERAVRZERO_NAO)) {
            if ($item->getIdrefevalorcalc() !== FpaitecalcfolhaEntity::IDREFEVALORCALC_VALOR
                && $item->getIdrefevalorcalc() !== FpaitecalcfolhaEntity::IDREFEVALORCALC_REFERENCIA_VALOR
            ) {
                $this->removeItem($item);
                return true;
            }
         }

         return false;
    }


    /**
     * Adiciona um Item de Calculo na lista.
     *
     * @param FpaeventomEntity $evento         Evento.
     * @param TkDateTime       $mes_ocorrencia Mes Concorrencia.
     * @param float            $valor          Valor.
     * @param integer          $referencia     Referencia.
     * @param integer          $contador       Contador.
     *
     * @return void
     *
     * @author  alessandrobartels
     */
    public function geraEvento(
        FpaeventomEntity $evento, TkDateTime $mes_ocorrencia = null, $valor, $referencia, $contador
    ){
        $nrevento = $evento->getNreventom();

        $posicao = $this->procuraEvento($evento, $mes_ocorrencia);
        /* $posicao = $this->procuraEvento($evento, null); */

        if ($posicao !== -1) {
            /** @var $item FpaitecalcfolhaEntity */
            $item = $this->getItem($posicao);

            // Criado metodo para resolver complexidade ciclomatica - alessandrobartels - 26/01/2012.
            $removeItem = $this->_verificaRemoveItem($valor, $evento->historico->getIdgeravrzero(), $item);
            if ($removeItem === true) {
               // tk_warning('passei '.$nrevento);
                return;
            }

            /*if (($valor === 0.0) && ($evento->historico->getIdgeravrzero() === FpaeventohEntity::IDGERAVRZERO_NAO)) {
                if ($item->getIdrefevalorcalc() !== FpaitecalcfolhaEntity::IDREFEVALORCALC_VALOR
                    && $item->getIdrefevalorcalc() !== FpaitecalcfolhaEntity::IDREFEVALORCALC_REFERENCIA_VALOR
                ) {
                    $this->removeItem($item);
                    return;
                }
            }*/

            if ($item->getIdrefevalorcalc() !== FpaitecalcfolhaEntity::IDREFEVALORCALC_VALOR
                && $item->getIdrefevalorcalc() !== FpaitecalcfolhaEntity::IDREFEVALORCALC_REFERENCIA_VALOR
                && $item->getIdrefevalorcalc() !== FpaitecalcfolhaEntity::IDREFEVALORCALC_VALOR_RECALCULO
                && $item->getIdrefevalorcalc() !== FpaitecalcfolhaEntity::IDREFEVALORCALC_REFERENCIA_VALOR_RECALCULO
            ) {

                $valor = Valor::arredonda(
                    $valor, $evento->historico->getIdtipoarredonda(), $evento->historico->getNrdigarredonda(),
                    $evento->historico->getVrlimitearred()
                );

                /* @todo parametrizar a precisao decimal */
                $item->setVreventofolha($valor);
            }

            if ($item->getIdrefevalorcalc() !== FpaitecalcfolhaEntity::IDREFEVALORCALC_REFERENCIA
                && $item->getIdrefevalorcalc() !== FpaitecalcfolhaEntity::IDREFEVALORCALC_REFERENCIA_VALOR
                && $item->getIdrefevalorcalc() !== FpaitecalcfolhaEntity::IDREFEVALORCALC_REFERENCIA_RECALCULO
                && $item->getIdrefevalorcalc() !== FpaitecalcfolhaEntity::IDREFEVALORCALC_REFERENCIA_VALOR_RECALCULO
            ) {
                /* @todo rever tipo de dado na tabela*/
                if ($referencia !== null) {
                    $item->setVrreffolha($referencia);
                }
            }

            if ($contador !== null) {
                $item->setQtocorproxperi($contador);
            }

            /* @todo Verificar nos demais metodos que trabalham com lista para atuar da mesma forma */
            /* $this->addItem($item, $posicao); */
        } else {
//            if ($nrevento == 60){
//                tk_warning('nao pode passar');
//            }
            if (($valor === 0.0)
                && ($evento->historico->getIdgeravrzero() === FpaeventohEntity::IDGERAVRZERO_NAO)
            ) {
                return;
            }

            if ($mes_ocorrencia === null) {
                $mes_ocorrencia = $evento->determinaDataOcorrencia();
            }

            /* @todo Teste para verificar em qual evento o nr calculo vem nulo. */
            if ($this->getNrcalculofolha() === null) {
                tk_warning('Gera evento => Nrevento: '.$nrevento);
            }

            $item = new self();
            /* @todo parametrizar a precisao decimal */

            $valor = Valor::arredonda(
                $valor, $evento->historico->getIdtipoarredonda(), $evento->historico->getNrdigarredonda(),
                $evento->historico->getVrlimitearred()
            );

            /*BORBA 22/10/2014 Verifica Piso e Teto*/
            if ($valor<$evento->historico->getVrpisoevento()) {
                $valor=$evento->historico->getVrpisoevento();
            } elseif ($valor>$evento->historico->getVrtetoevento()) {
                $valor=$evento->historico->getVrtetoevento();
            }


            $item->setVreventofolha($valor);
            $item->setVrreffolha($referencia);
            $item->setDtocorrencia($mes_ocorrencia);
            $item->setQtocorproxperi($contador);
            $item->setNreventom($nrevento);
            $item->setNrcalculofolha($this->getNrcalculofolha());
            $item->setIdevendemoncalc($evento->historico->getIdeventodemonst());
            $item->setNrvinculom($this->getNrvinculom());
            $item->setIdrefevalorcalc(self::IDREFEVALORCALC_NORMAL);
            $item->setNrorg($this->getNrorg());
            $this->addItem($item);
        }

    }


    /**
     * Subtrai valores entre 2 itens de calculo.
     *
     * @param FpaeventomEntity $evento_origem  Evento origem.
     * @param FpaeventomEntity $evento_destino Evento destino.
     * @param TkDateTime       $dt_ocorrencia  Data de ocorrencia.
     * @param Boolean          $aceitanegativo Aceita valor negativo
     *
     * @return void
     *
     * @author alessandrobartels
     */
    public function subtraiValor(
        FpaeventomEntity $evento_origem, FpaeventomEntity $evento_destino, TkDateTime $dt_ocorrencia, $aceitanegativo=true
    ) {

        $valor_origem = $this->retornaValorPorEvento($evento_origem, $dt_ocorrencia);

        $valor_destino = $this->retornaValorPorEvento($evento_destino, $dt_ocorrencia);
        $diferenca = $valor_origem - $valor_destino;
        if ($aceitanegativo==false && $diferenca<0){
            $diferenca=0;
        }

        $this->geraEvento($evento_destino, $dt_ocorrencia, $diferenca, null, null);

    }


    /**
     * Retorna valor do evento.
     *
     * @param FpaeventomEntity $evento        Evento.
     * @param TkDateTime       $dt_ocorrencia Data de ocorrencia.
     *
     * @return float
     *
     * @author alessandrobartels
     */
    public function retornaValorPorEvento(FpaeventomEntity $evento, TkDateTime $dt_ocorrencia = null) {
        $posicao = $this->procuraEvento($evento, $dt_ocorrencia);
        if ($posicao !== -1) {
            $valor = $this->retornaValor($posicao);
        } else {
            $valor = 0;
        }

        return $valor;
    }


    /**
     * Retorna contador do evento.
     *
     * @param FpaeventomEntity $evento        Instancia de FpaeventomEntity.
     * @param TkDateTime       $dt_ocorrencia Data de ocorrencia.
     *
     * @return integer
     *
     * @author alessandrobartels 30/08/2011
     */
    public function retornaContadorPorEvento(FpaeventomEntity $evento, TkDateTime $dt_ocorrencia = null) {
        $posicao = $this->procuraEvento($evento, $dt_ocorrencia);
        if ($posicao !== -1) {
            $contador = $this->retornaContador($posicao);
        } else {
            $contador = 1;
        }

        return $contador;
    }


    /**
     * Retorna referencia do evento.
     *
     * @param FpaeventomEntity $evento        Instancia de FpaeventomEntity.
     * @param TkDateTime       $dt_ocorrencia Data de ocorrencia.
     *
     * @return integer
     *
     * @author alessandrobartels 11/08/2011
     */
    public function retornaReferenciaPorEvento(FpaeventomEntity $evento, TkDateTime $dt_ocorrencia = null) {
        $posicao = $this->procuraEvento($evento, $dt_ocorrencia);
        if ($posicao !== -1) {
            $referencia = $this->retornaReferencia($posicao);
        } else {
            $referencia = 0;
        }

        return $referencia;
    }


    /**
     * Procura a posicao do item de calculo pelo numero do evento.
     *
     * @param FpaeventomEntity $evento         Evento.
     * @param TkDateTime       $mes_ocorrencia Mes Concorrencia.
     *
     * @return integer
     *
     * @author  alessandrobartels
     */
    public function procuraEvento(FpaeventomEntity $evento, $mes_ocorrencia = null){
        $itens = $this->getItems();
        $dt_item = new TkDateTime();

        /* alessandrobartels - 29/08/2011 Verificando a data dos itens na lista somente pelo numero do mes
         *                                e nao pela data cheia.
         */
        $mes = new TkDateTime($mes_ocorrencia);
        $numero_mes = $mes->toValue(TkDateTime::MONTH);
        // $numero_ano = $mes->toValue(TkDateTime::YEAR);
        /** @var $item FpaitecalcfolhaEntity */

        foreach ($itens as $indice => $item) {
            $nrevento_item = $item->getNreventom();
//            if (!!$evento === false) {
//                debug_print_backtrace();
//                var_dump($evento);
//                die();
//            }

            $nrevento_evento = $evento->getNreventom();
            $dt_item->set($item->getDtocorrencia(), TkDateTime::FORMAT_DATE);
            /*   if ($mes_ocorrencia === null) {
                $mes_ocorrencia = $evento->determinaDataOcorrencia();
            }
             */
            /* @todo Verificar - nem toda data de mes de ocorrencia vem como TkDateTime.
             *       Assim que arrumar retirar a linha abaixo.
             */
			 
//if (($evento->getNreventom() == 3050) && ($nrevento_item == 3050)) {			
//tk_abort($evento->getNreventom(). " - ". $item->getDtocorrencia()." - ".$nrevento_item. "  - ".$nrevento_evento. " - ".$mes_ocorrencia);
//}			 
			 
            if ($mes_ocorrencia !== null) {
                if (((int)$nrevento_item === (int)$nrevento_evento)
                    && ($dt_item->equals($numero_mes, TkDateTime::MONTH) === true)
                   // && ($dt_item->equals($numero_ano, TkDateTime::YEAR) === true)
                ) {
                    unset($mes);
                    return $indice;
                }
            } else {
                if ((int)$nrevento_item === (int)$nrevento_evento) {
                    unset($mes);
                    return $indice;
                }
            }
        }

        return -1;
    }


    /**
     * Verifica a existencia de evento de liquido na lista de itens de calculo.
     *
     * @param FpaassoceventoEntity $assoc_eventos Associacao de eventos.
     * @param FpaeventomEntity     $eventom       Evento.
     * @param integer              $nrOrg         Numero da Organizacao.
     *
     * @return boolean
     *
     * @author  alessandrobartels
     */
    public function verificaLiquido(FpaassoceventoEntity $assoc_eventos, FpaeventomEntity $eventom, $nrOrg){
        $evento_aux = $assoc_eventos->retornaEventoAssociado(FpatipoassoceveEntity::ASSOC_LIQUIDO, $eventom, $nrOrg);
        $itens = $this->retornaItens($evento_aux);
        if (empty($itens) === false) {
            return true;
        } else { //BORBA 27/09/2013 Verifica se tem líquido de rescisão
            $evento_aux = $assoc_eventos->retornaEventoAssociado(FpatipoassoceveEntity::ASSOC_LIQUIDO_RESCISAO, $eventom, $nrOrg);
            $itens = $this->retornaItens($evento_aux);
            if (empty($itens) === false) {
                return true;
            } else{
                return false;
            }
        }
    }


    /**
     * Enter description here ...
     *
     * @param FpaassoceventoEntity $assoc_eventos Instancia de FpaassoceventoEntity.
     * @param FpaeventomEntity     $eventom       Instancia de FpaeventomEntity.
     * @param integer              $nrOrg         Numero da Organizacao.
     *
     * @return boolean
     */
    public function verificaLiquidoParcial(FpaassoceventoEntity $assoc_eventos, FpaeventomEntity $eventom, $nrOrg){
        $evento_aux = $assoc_eventos->retornaEventoAssociado(
            FpatipoassoceveEntity::ASSOC_LIQUIDO_PARCIAL, $eventom, $nrOrg
        );
        $itens = $this->retornaItens($evento_aux);
        if (empty($itens) === false) {
            return true;
        } else {
            return false;
        }
    }


    /**
     * Procura itens de calculo na lista pelo evento parametrizado.
     *
     * @param FpaeventomEntity $evento Evento.
     *
     * @return array
     *
     * @author  alessandrobartels
     */
    public function retornaItens(FpaeventomEntity $evento){
        $posicoes = array();
        $itens = $this->getItems();
        foreach ($itens as $indice => $item) {
            $item->getNreventom();
            $evento->getNreventom();
            if ($item->getNreventom() === $evento->getNreventom()) {
               // tk_warning($item->getNreventom().'  '.$evento->getNreventom());
                $posicoes[] = $indice;
            }
        }
        return $posicoes;
    }


    /**
     * Adiciona na lista os itens de calculo do vinculo parametrizado.
     *
     * @param integer $nr_org     Numero Organizacao.
     * @param integer $nr_vinculo Numero Vinculo.
     * @param integer $nr_calculo Numero Calculo.
     *
     * @return integer
     *
     * @author  alessandrobartels
     */
    public function retornaItensPorVinculo($nr_org, $nr_vinculo, $nr_calculo){

        if ($nr_calculo === null) {
            tk_abort('Numero de calculo nulo.', 'warning');
//            die();
        }

        $this->resetItems();
        $contador = 0;

        /* Setando vinculo corrente e calculo corrente*/
        $this->setNrvinculom($nr_vinculo);
        $this->setNrcalculofolha($nr_calculo);
        $this->setNrorg($nr_org);

        $resultado = Fpaitecalcfolha::retornaItem($nr_vinculo, $nr_calculo, $nr_org);

        if (empty($resultado) === false) {
            foreach ($resultado as $entity_obj) {
                $this->addItem($entity_obj);
                $contador++;
            }
        }

        return $contador;
    }


    /**
     * Retorna array com itens de calculo de TOTAIS.
     *
     * @param FpaeventomEntity $eventom Evento.
     *
     * @return array
     *
     * @author  alessandrobartels
     */
    public function retornaTotais(FpaeventomEntity $eventom){
        $totais = array(
            FpaeventomEntity::IDTIPOEVENTO_PROVENTO => 0, FpaeventomEntity::IDTIPOEVENTO_DESCONTO => 0,
            FpaeventomEntity::IDTIPOEVENTO_ENCARGO => 0, FpaeventomEntity::IDTIPOEVENTO_TOTAL => 0,
        );
        foreach ($this->getItems() as $item) {
            $evento = $eventom->retornaEvento($item->getNreventom());
            if (is_object($evento) === true) {
                if ($item->getIdevendemoncalc() === self::IDEVENDEMONCALC_NAO) {
                    $totais[$evento->getIdtipoevento()] += (float)$item->getVreventofolha();
                }
            } else {
                tk_warning('Calculo de Totais: Evento '.$item->getNreventom().' nao encontrado para a Org.');
            }
        }

        return $totais;
    }


    /**
     * Retorna o calculo dos valores dos eventos de uma determinada incidencia.
     *
     * @param integer          $tipo_incidencia         Tipo Incidencia.
     * @param string           $data_competencia        Data Competencia.
     * @param FpaeventomEntity $eventom                 Evento.
     * @param boolean          $referenciaOcorrencia    referenciaOcorrencia.

     * @return integer|array
     *
     * @author  alessandrobartels
     *          alessandrobartels 25/02/2012 - Comparacao para incrementar/decrementar $base passa a ser pelo
     *                                         IDINCIDEVENTO  (N, +, -)
     *          brunoborba 20/03/2014 Correção do array acumulador que e documentação das alterações feitas pelo jorge
     */
    public function calculaIncidencias($tipo_incidencia, $data_competencia, FpaeventomEntity $eventom,
                                       $referenciaOcorrencia = false)
    {
        $base = 0.0;
        $this->_totalReferencias = 0;
        $bases = array();
        /** @var $item \FpaitecalcfolhaEntity */
        foreach ($this->getItems() as $item) {
            $evento = $eventom->retornaEvento($item->getNreventom());
            if (is_object($evento) === true) {
                $aux_incide_evento = FpaincideventoEntity::incide(
                    $tipo_incidencia, $evento->getNreventom(), $data_competencia
                );
                /**
                 *
                 * Realiza comparacao direto com o vetor de incidencias e nao mais com o evento.
                 *  if ($aux_incide_evento === true) {
                 *  if ($evento->getIdtipoevento() === FpaeventomEntity::IDTIPOEVENTO_PROVENTO) {
                 *      $base += $item->getVreventofolha();
                 *  } else if ($evento->getIdtipoevento() === FpaeventomEntity::IDTIPOEVENTO_DESCONTO) {
                 *      $base -= $item->getVreventofolha();
                 *  }
                 * } else {
                 *   false;
                 *    tk_warning('Nao existe associacao de incidencia para o evento '.$evento->getNreventom());.
                 * }
                 */
                if ($aux_incide_evento === self::MAIS) {
                    $base += $item->getVreventofolha();
                    $this->_totalReferencias += $item->getVrreffolha();
                } elseif ($aux_incide_evento === self::MENOS) {
                    $base -= $item->getVreventofolha();
                    $this->_totalReferencias -= $item->getVrreffolha();
                }
                //JORGE 11/11/2013 - Quando a data de ocorrencia for diferente da atual criar uma nova posicao no array para acumular
                /*BORBA 20/03/2014 - O array acumulador $bases utilizava a $base previamente acumulada gerando duplicidade na soma,
                                     a solução foi nao acumular a $base ao array $basese sim o $item->getVreventofolha() */
                if ($referenciaOcorrencia && $aux_incide_evento === self::MAIS) {
                    $dtocorrencia = new \TkDateTime($item->getDtocorrencia(), \TkDateTime::FORMAT_DATE);
                    $bases[$dtocorrencia->getTimestamp()] = isset($bases[$dtocorrencia->getTimestamp()]) ?
                        $bases[$dtocorrencia->getTimestamp()] + $item->getVreventofolha() : $item->getVreventofolha();
                }elseif ($referenciaOcorrencia && $aux_incide_evento === self::MENOS) {
                    $dtocorrencia = new \TkDateTime($item->getDtocorrencia(), \TkDateTime::FORMAT_DATE);
                    $bases[$dtocorrencia->getTimestamp()] = isset($bases[$dtocorrencia->getTimestamp()]) ?
                        $bases[$dtocorrencia->getTimestamp()] - $item->getVreventofolha() : $item->getVreventofolha();
                }
            } else {
                tk_warning('Calculo de Incidencias: Evento '.$item->getNreventom().' nao encontrado para a Org.');
            }
        }
        return ($referenciaOcorrencia) ? $bases : $base;
    }


    /**
     * Retorna a soma das referencias dos eventos de uma determinada incidencia.
     *
     * Nao verifica o tipo do evento (PROVENTO, DESCONTO), por se tratar de referencia.
     * Executa a soma sem verificar o tipo de referencia (DIA, HORA, PERCENTUAL)
     *
     * @param integer          $tipo_incidencia  Tipo Incidencia.
     * @param string           $data_competencia Data Competencia.
     * @param FpaeventomEntity $eventom          Evento.
     *
     * @return integer
     *
     * @author  brunofeliciano - 17/08/2011
     */
    public function calculaReferenciaIncidencias($tipo_incidencia, $data_competencia, FpaeventomEntity $eventom){

        $ref = 0;
        foreach ($this->getItems() as $item) {

            $evento = $eventom->retornaEvento($item->getNreventom());

            $aux_incide_evento = FpaincideventoEntity::incide(
                $tipo_incidencia, $evento->getNreventom(), $data_competencia
            );
            tk_warning($aux_incide_evento);
            if ($aux_incide_evento === true) {
                /* if ($evento->getIdtipoevento() === FpaeventomEntity::IDTIPOEVENTO_PROVENTO) { */
                $ref += $item->getVrreffolha();
                /*} else if ($evento->getIdtipoevento() === FpaeventomEntity::IDTIPOEVENTO_DESCONTO) {
                    $base -= $item->getVrreffolha();
                }*/
            }
        }

        return $ref;
    }


    /**
     * Enter description here ...
     *
     * @return boolean
     *
     * @author alessandrobartels
     */
    public function apagaItemCalculo(){

        $remove = false;
        foreach ($this->getItems() as $item) {
            $this->removeItem($item);
            $remove = true;
        }

        return $remove;
    }


    /**
     * Enter description here ...
     *
     * @param FpaeventomEntity $eventom Evento.
     *
     * @return void
     *
     * @author alessandrobartels
     */
    public function executaRetrocesso(FpaeventomEntity $eventom){
        foreach ($this->getItems() as $item) {
            $evento = $eventom->retornaEvento($item->getNreventom());
            if ($evento->historico->getIdretrocedevent() === FpaeventohEntity::IDRETROCEDEVENT_SEMPRE) {
                if ($item->getIdrefevalorcalc() !== self::IDREFEVALORCALC_NORMAL) {
                    switch ($item->getIdrefevalorcalc()){
                    case self::IDREFEVALORCALC_REFERENCIA /*LANC_REFERENCIA*/ :
                        $item->setVreventofolha(0); break;
                    case self::IDREFEVALORCALC_VALOR /*LANC_VALOR*/ :
                        $item->setVrreffolha(0); break;
                    }
                } else {
                    if ($evento->historico->getIdtipoocorreven() === FpaeventohEntity::IDTIPOOCORREVEN_SISTEMA) {
                        $this->removeItem($item);
                    } else {
                        /*echo $item->nreventom."   ".$item->getVrreffolha().'<br/>';*/
                        // if ($item->getVrreffolha() !== 0.0) { RETIRADO TESTE DE TIPO BORBA 27/02/2013
                        if ($item->getVrreffolha() != 0.0) {
                            $item->setVreventofolha(0);
                            /*echo $item->getVrreffolha().'vai zerar valor<br/>';*/
                        // } elseif (($item->getVreventofolha() === 0.0) && ($item->getVrreffolha() === 0.0)) { RETIRADO TESTE DE TIPO BORBA 27/02/2013
                        } elseif (($item->getVreventofolha() == 0.0) && ($item->getVrreffolha() == 0.0)) {
                            /*echo $item->getVrreffolha().'vai remover<br/>';*/
                            $this->removeItem($item);
                        }
                    }
                }
            }
        }
    }


    /**
     * Metodo que transfere os eventos para o item calculo que sera recalculado.
     *
     * @param FpaitecalcfolhaEntity $itens_origem Itens Origem.
     * @param FpaeventomEntity      $eventom      Evento.
     *
     * @return void
     *
     * @author alessandrobartels
     */
    public function preparaRecalculo(FpaitecalcfolhaEntity $itens_origem, FpaeventomEntity $eventom){
        /*Normalmente estes eventos restaram apos um retrocesso.*/
        /*$this->listaItemCalculo("MOVIMENTO ORIGEM",$eventom);*/
        foreach ($itens_origem->getItems() as $indice => $item) {
            $evento_aux = $eventom->retornaEvento($item->getNreventom());
            $referencia = $itens_origem->retornaReferencia($indice);
            $valor_item = $itens_origem->retornaValor($indice);
            $dtocorrencia = $itens_origem->retornaDtocorrencia($indice);
            $this->geraEvento($evento_aux, $dtocorrencia, $valor_item, $referencia, 1);
        }
    }


    /**
     * Enter description here ...
     *
     * @param FpaitecalcfolhaEntity $itens_origem Itens de origem.
     * @param FpaeventomEntity      $eventom      Evento.
     *
     * @return void
     *
     * @author alessandrobartels
     */
    public function calculaDiferencaRecalculo(FpaitecalcfolhaEntity $itens_origem, FpaeventomEntity $eventom){


        foreach ($itens_origem->getItems() as $indice => $item_origem) {		

            $evento_aux = $eventom->retornaEvento($item_origem->getNreventom());
            $referencia = $itens_origem->retornaReferencia($indice);
            $valor_item = $itens_origem->retornaValor($indice);
            $dtocorrencia = $itens_origem->retornaDtocorrencia($indice);
            $posicao = $this->procuraEvento($evento_aux, null);
            /*echo $posicao."     ".$item_origem->nreventom."   ".$valor_item.'<br/>';*/		
            if ($posicao !== -1) {
                $item_corrente = $this->getItem($posicao);
                /*echo "valor:  ".$item_corrente->getVreventofolha().'<br/>';*/
                /*$item_corrente->vreventofolha -= $valor_item;*/

                if ($item_corrente->getIdrefevalorcalc() !== FpaitecalcfolhaEntity::IDREFEVALORCALC_VALOR_RECALCULO
                    && $item_corrente->getIdrefevalorcalc() !== FpaitecalcfolhaEntity::IDREFEVALORCALC_REFERENCIA_VALOR_RECALCULO
                ) {
                    $item_corrente->setVreventofolha($item_corrente->getVreventofolha() - $valor_item);


                    //BORBA 18/05/2015 Caso o valor do evento ao piso e ao teto.
                    if ($item_corrente->getVreventofolha() < $evento_aux->historico->getVrpisoevento()) {
                        $item_corrente->setVreventofolha($evento_aux->historico->getVrpisoevento());
                    } elseif ($item_corrente->getVreventofolha()> $evento_aux->historico->getVrtetoevento()) {
                        $item_corrente->setVreventofolha($evento_aux->historico->getVrtetoevento());
                    }
                }
                /*echo $item_corrente->nreventom."  valor:  ".$item_corrente->getVreventofolha()."<br/>";*/
                if (($item_corrente->getVreventofolha() === 0.0)
                    && ($evento_aux->historico->getIdgeravrzero() === FpaeventohEntity::IDGERAVRZERO_NAO)
                ) {
                    $this->removeItem($item_corrente);
                } else {
                    false;
                    /* atualizacao - nao precisa add item - alessandro 25/08/2011.
                     * $this->addItem($item_corrente, $item_corrente->getNritemcalcfolha());
                     */
                }
            }
        }
    }


    /**
     * Enter description here ...
     *
     * @param GpevinculomEntity $vinculo                Vinculo.
     * @param string            $lista_nr_calculo_folha Lista Numero Calculo Folha.
     * @param string            $lista_nr_eventos       Lista Numero Eventos.
     *
     * @return array
     *
     * @author alessandrobartels
     */
    public function totalizaItensHistorico(
        GpevinculomEntity $vinculo, $lista_nr_calculo_folha, $lista_nr_eventos
    ){
        $lista = array();

        if (empty($lista_nr_eventos) === true) {
            return;
        }

        if (empty($lista_nr_calculo_folha) === true) {
            return;
        }

        $this->retornaItensPorPeriodo(
            $vinculo->getNrorg(), $vinculo->getNrvinculom(), $lista_nr_calculo_folha, $lista_nr_eventos
        );

        $nr_evento_ant = -1;
        //25-02-2015 brunoborba Conversão da referencia do evento hora/minuto para decimal evento a evento antes de sumarizar
        $eventom = new FpaeventomEntity(null, \FPA::getNrorgTrabalho(), \FPA::getNrorgPadrao(), \FPA::getDtCompetencia());

        foreach ($this->getItems() as $item) {
            $index = (int)$item->getNreventom();
            //Marcelo 16-12-2014 No caso de adiantamento de salario, deve-se considerar somente o valor da maior ocorrencia (sem somar os valores).
            if ($item->getNreventom() == FpaTipoassoceveEntity::ASSOC_ADIANT_13SAL) {
                if (array_key_exists($index, $lista)) {
                    continue;
                } else {
                    $lista[$index][self::REFERENCIA] = $item->getVrreffolha();
                    $lista[$index][self::VALOR] = $item->getVreventofolha();
                    continue;
                }
            }
            //25-02-2015 brunoborba Conversão da referencia do evento hora/minuto para decimal evento a evento antes de sumarizar
            $evento = $eventom->retornaEvento($index);
            if($evento->historico->getIdunidrefereven()==FpaeventohEntity::IDUNIDREFEREVEN_HORA_MINUTO){
                $item->setVrreffolha((\FPA::converteHoraDecimal($item->getVrreffolha())));
            };

            if ($item->getNreventom() !== $nr_evento_ant) {
                if ($nr_evento_ant === -1) {
                    $lista[$index][self::REFERENCIA] = $item->getVrreffolha();
                    $lista[$index][self::VALOR] = $item->getVreventofolha();
                } else {
                    $lista[$index][self::REFERENCIA] += $item->getVrreffolha();
                    $lista[$index][self::VALOR] += $item->getVreventofolha();
                }
            } else {
                $lista[$index][self::REFERENCIA] += $item->getVrreffolha();
                $lista[$index][self::VALOR] += $item->getVreventofolha();
            }
            $nr_evento_ant = (int)$item->getNreventom();
        }

        return $lista;
    }


    /**
     * Enter description here ...
     *
     * @param string           $mes_ocorrencia    Mes Ocorrencia.
     * @param FpaeventomEntity $eventom           Evento.
     * @param integer          $itemDemonstrativo Item Demonstrativo.
     *
     * @return array
     *
     * @author alessandrobartels
     */
    public function retornaArrayItemCalculo($mes_ocorrencia, FpaeventomEntity $eventom, $itemDemonstrativo = null){

        $lista = array();
        $contador = 1;
        $itens	= $this->getItems();
        if (count($itens) === 0) {
            return null;
        }

        $dataItem = new TkDateTime($mes_ocorrencia, TkDateTime::FORMAT_DATE);
        $dt_ocorrencia = new TkDateTime();

        foreach ($itens as $key => $item) {
            $evento = $eventom->retornaEvento($item->getNreventom());
            if (is_object($evento) === true) {
                $dataItem->set($item->getDtocorrencia());
                if ($mes_ocorrencia->equals($dataItem) === true) {
                    $mesOcorrencia = null;
                } else {
                    $dt_ocorrencia->set($item->getDtocorrencia(), TkDateTime::FORMAT_DATE);
                    $mesOcorrencia = $dt_ocorrencia->toString(TkDateTime::FORMAT_DATE);
                }

                $itemDemonstrativo->addEvento(
                    $mesOcorrencia,
                    $evento->getIdtipoevento(),
                    $item->getNreventom(),
                    $evento->historico->getNmeventoh(),
                    $evento->historico->getIdunidrefereven(),
                    $item->getVrreffolha(),
                    $item->getIdevendemoncalc(),
                    $item->getVreventofolha(),
                    $evento->historico->getNrordimpreseven(),
                    $evento->getOrdemTipoEvento($evento->getIdtipoevento())
                );
            } else {
                tk_warning('Persiste Item de Calculo: Evento '.$item->getNreventom().' nao encontrado para a Org.');
            }
        }

        return $itemDemonstrativo;
    }


    /**
     * Enter description here ...
     *
     * @param integer $nr_org           Numero Organizacao.
     * @param integer $nr_vinculo       Numero Vinculo.
     * @param string  $lista_nr_calculo Lista Numero Calculo.
     * @param string  $lista_nr_evento  Lista Numero Evento.
     *
     * @return void
     *
     * @author alessandrobartels
     *         alessandrobartels - 28/06/2011 - Modificado a condicao. O metodo retornaItemPeriodo retorna um array.
     */
    public function retornaItensPorPeriodo($nr_org, $nr_vinculo, $lista_nr_calculo, $lista_nr_evento){

        $this->resetItems();
        $resultado = Fpaitecalcfolha::retornaItemPeriodo($nr_org, $nr_vinculo, $lista_nr_calculo, $lista_nr_evento);
        if (count($resultado) > 0) {
            foreach ($resultado as $entity_obj) {
                $this->addItem($entity_obj);
            }
        }
    }


    /**
     * Inclui um novo itecalcfolha.
     *
     * @param integer    $nrCalculo             Numero da Calculo.
     * @param integer    $nrVinculo             Numero do vinculo.
     * @param integer    $nrEvento              Numero do Evento.
     * @param integer    $nrOrg                 Numero da Organizacao.
     * @param string     $idEventoDemonstrativo Flag de evento demonstrativo.
     * @param float      $referencia            Valor da referencia.
     * @param float      $valor                 Valor.
     * @param string     $idReferenciaValor     Flag de referencia/valor.
     * @param integer    $qtdOcorrencia         Quantidade ocorrencia.
     * @param TkDateTime $dtOcorrencia          Data de ocorrencia.
     *
     * @return void
     *
     * @author luistavares - 30/08/2011
     */
    public function incluiIteCalcFolha(
        $nrCalculo, $nrVinculo, $nrEvento, $nrOrg, $idEventoDemonstrativo, $referencia, $valor,
        $idReferenciaValor, $qtdOcorrencia, $dtOcorrencia
    ){

        $item = new self();

        $item->setNrcalculofolha($nrCalculo);
        $item->setNrvinculom($nrVinculo);
        $item->setNreventom($nrEvento);
        $item->setNrorg($nrOrg);
        $item->setIdevendemoncalc($idEventoDemonstrativo);
        $item->setVrreffolha($referencia);
        $item->setVreventofolha($valor);
        $item->setQtocorproxperi($qtdOcorrencia);
        $item->setIdrefevalorcalc($idReferenciaValor);
        $item->setDtocorrencia($dtOcorrencia);

        $this->addItem($item);
    }


    /**
     * Enter description here ...
     *
     * @param integer $nrOrg          Numero da Organizacao.
     * @param integer $nrVinculo      Numero do Vinculo.
     * @param integer $nrCalculoFolha Numero do Calculo de Folha.
     * @param integer $nrTipoAssoc    Numero do Tipo de Asscociacao de Evento.
     *
     * @return boolean
     */
    public function pesquisaEvento($nrOrg, $nrVinculo, $nrCalculoFolha, $nrTipoAssoc){
        $resultado = Fpaitecalcfolha::pesquisaEventoPorTipoAssoc(
            $nrOrg, $nrVinculo, $nrCalculoFolha, $nrTipoAssoc
        );

        if (count($resultado) > 0) {
            return true;
        } else {
            return false;
        }

    }

    public function insereItemCalculoFolha($nrorg, $nrcalculo, $nrvinculo, $nrevento, $dtocorrencia, $idevendemoncalc,
                                           $referencia, $valor, $qtdOcorrencia, $idReferenciaValor)
    {
        $this->setNrorg($nrorg);
        $this->setNrcalculofolha($nrcalculo);
        $this->setNrvinculom($nrvinculo);
        $this->setNreventom($nrevento);
        $this->setDtocorrencia($dtocorrencia);
        $this->setIdevendemoncalc($idevendemoncalc);
        $this->setVrreffolha($referencia);
        $this->setVreventofolha($valor);
        $this->setQtocorproxperi($qtdOcorrencia);
        $this->setIdrefevalorcalc($idReferenciaValor);
    }

    /**
     * @return FpaeventomEntity
     */
    public function getEvento()
    {
        if (!$this->evento) {
            $this->evento = FpaeventomEntity::getEvento($this->getNreventom());
        }
        return $this->evento;
    }

    public static function getItensCalculoFolha($nrorg, $nrcalculofolha)
    {
        $fpaitecalcfolha = self::getEntitiesByWhere(
            'NRORG = ? AND NRCALCULOFOLHA = ?', array($nrorg, $nrcalculofolha)
        );
        return $fpaitecalcfolha;
    }

    public static function getItemCalculoPorChaveUnica($nrorg, $nrcalculo, $nrvinculom, $nreventom,
                                                                       $dtocorrencia)
    {
        $params = array($nrorg, $nrcalculo, $nrvinculom, $nreventom, $dtocorrencia);
        $fpaitecalcfolha = self::getEntityByWhere(
            'NRORG = ? AND NRCALCULOFOLHA = ? AND NRVINCULOM = ? AND NREVENTOM = ? AND DTOCORRENCIA = ?', $params
        );
        return $fpaitecalcfolha;
    }

    /* @todo BORBA - 10/06/2013 método temporário*/
    public static function getItemCalculoPorVinculoDtOcor($nrorg, $nrvinculom, $nreventom,
                                                       $dtocorrencia)
    {
        $params = array($nrorg, $nrvinculom, $nreventom, $dtocorrencia);
        $fpaitecalcfolha = self::getEntityByWhere(
            'NRORG = ? AND NRVINCULOM = ? AND NREVENTOM = ? AND DTOCORRENCIA = ?', $params
        );
        return $fpaitecalcfolha;
    }

    /**
     * Altera o atributo de um item caso este existe na lista de itens
     * Criado para transportar o avo integral para o cálculo Atendendo à solicitação 153857
     *
     * @param FpaeventomEntity      $eventom        Evento.
     * @param string                $atributo       Atributo.
     * @param string                $valorAtributo  Valor do Atributo
     * @return void
     *
     * @author brunoborba 05/01/2015
     */
    public function alteratributoitem(FpaeventomEntity $eventom,$atributo,$valorAtributo){

    /*@todo definir atributo que será alterado para tornar o método 'inteligente'*/

        foreach ($this->getItems() as $indice => $item_corrente) {
            $posicao = $this->procuraEvento($eventom, null);
            if ($posicao !== -1) {									
                $item_corrente = $this->getItem($posicao);

				//Marcelo 03-02-2015 O valor da referência deve ser atribuido somente se o valor da mesma não tiver sido forçado pelo operador.
                if ($item_corrente->getIdrefevalorcalc() !== FpaitecalcfolhaEntity::IDREFEVALORCALC_REFERENCIA
                    && $item_corrente->getIdrefevalorcalc() !== FpaitecalcfolhaEntity::IDREFEVALORCALC_REFERENCIA_VALOR
                ) {							
					$item_corrente->setVrreffolha($valorAtributo);
					/*tk_warning('Pos->'.$posicao.'   NREVEN->'.$item_corrente->getNreventom().'   ATRIBUTO->'.$atributo.'   VRATRIBUTO->'.$valorAtributo);*/
				}	
            }
        }
    }

    /**
     * Retira o item da lista caso este exista na lista
     * Criado para atender à solicitação 153857
     *
     * @param FpaeventomEntity      $eventom        Evento.
     * @return void
     *
     * @author brunoborba 05/01/2015
     */
    public function retiraitem(FpaeventomEntity $eventom){

        foreach ($this->getItems() as $indice => $item_corrente) {
            $posicao = $this->procuraEvento($eventom, null);
            if ($posicao !== -1) {
                $item_corrente = $this->getItem($posicao);
                $this->removeItem($item_corrente);
                /*tk_warning('Pos->'.$posicao.'   NREVEN->'.$item_corrente->getNreventom().'   ATRIBUTO->'.$atributo.'   VRATRIBUTO->'.$valorAtributo);*/
            }
        }
    }

}

