/******************************************************************************/
/* Sistemas Integrados TECNISA                                                */
/*                                                                            */
/* Compras e Suprimentos                                                      */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/* Nome: EVOLUCAO_PRECOS                                                      */
/*                                                                            */
/* Descrio:                                                                 */
/*   Calcula aos demais periodos e calcula a variacao entre cada periodo.     */
/*                                                                            */
/* Criao:                                                                   */
/*    05/06/2000 - Yris L. de Souza                                          */
/*                                                                            */
/* Manutenes:                                                               */
/*   /  /   -                                                                 */
/*                                                                            */
/*                                                                            */
/******************************************************************************/

/******************************************************************************/
/******************************************************************************/


IF EXISTS (SELECT * FROM SYSOBJECTS O, SYSUSERS U
                    WHERE
	        O.NAME = 'EVOLUCAO_PRECOS'  AND
	        TYPE = 'P'                  AND
	        U.UID = O.UID               AND
	        U.NAME = USER)

  DROP PROCEDURE EVOLUCAO_PRECOS
  GO


CREATE PROCEDURE EVOLUCAO_PRECOS  @P_NREVOLPRECO VARCHAR(10),
                                  @P_DTINI       DATETIME,
				  @P_DTFIN       DATETIME,
			          @P_NRPERI      NUMERIC,
 				  @P_CDPRODUTO   VARCHAR(10)
AS
 DECLARE @V_NRCONTA      VARCHAR(10);
 DECLARE @V_NRPERIODOS   NUMERIC;
 DECLARE @V_CDPRODUTO    VARCHAR(10);
 DECLARE @V_CDPRODUTOANT VARCHAR(10);
 DECLARE @V_DTINIPEREVOL DATETIME;
 DECLARE @V_VRPRECOEVOL  NUMERIC;
 DECLARE @V_VRPERCEVOL   NUMERIC;
 DECLARE @V_VRPRECAUX    NUMERIC;
 DECLARE @V_TCDPRODUTO   VARCHAR(01);
 DECLARE @V_DATAINI      DATETIME;
 DECLARE @V_DATAFIN      DATETIME;

-- CURSOR COM OS PERIODOS DE ACORDO COM A PERIODICIDADE E NUMERO
-- DE PERIODOS

DECLARE C_DATAS CURSOR FOR
SELECT
 CONVERT(DATETIME,@P_DTINI) DATAINI,
 CONVERT(DATETIME,@P_DTINI) + (@P_NRPERI * 1) - 1 DATAFIN
FROM
 PARAMGERAL
WHERE
 1 <= @V_NRPERIODOS
UNION
SELECT
 CONVERT(DATETIME,@P_DTINI) + (@P_NRPERI * 1) DATAINI,
 CONVERT(DATETIME,@P_DTINI) + (@P_NRPERI * 2) - 1 DATAFIN
FROM
 PARAMGERAL
WHERE
 2 <= @V_NRPERIODOS
UNION
SELECT
 CONVERT(DATETIME,@P_DTINI) + (@P_NRPERI * 2) DATAINI,
 CONVERT(DATETIME,@P_DTINI) + (@P_NRPERI * 3) - 1 DATAFIN
FROM
 PARAMGERAL
WHERE
 3 <= @V_NRPERIODOS
UNION
SELECT
 CONVERT(DATETIME,@P_DTINI) + (@P_NRPERI * 3) DATAINI,
 CONVERT(DATETIME,@P_DTINI) + (@P_NRPERI * 4) - 1 DATAFIN
FROM
 PARAMGERAL
WHERE
 4 <= @V_NRPERIODOS
UNION
SELECT
 CONVERT(DATETIME,@P_DTINI) + (@P_NRPERI * 4) DATAINI,
 CONVERT(DATETIME,@P_DTINI) + (@P_NRPERI * 5) - 1 DATAFIN
FROM
 PARAMGERAL
WHERE
 5 <= @V_NRPERIODOS;

-- CURSOR COM TODOS OS PRODUTOS SELECIONADOS

DECLARE C_PRODUTO CURSOR FOR
SELECT
  DISTINCT
  CDPRODUTO
 FROM
  TRABEVOLPREC
 WHERE
  NREVOLPRECO = @P_NREVOLPRECO AND
  (CDPRODUTO   = @P_CDPRODUTO OR @V_TCDPRODUTO = 'T');

DECLARE C_TRABEVOL CURSOR FOR
SELECT
  CDPRODUTO,
  DTINIPEREVOL,
  VRPRECOEVOL
 FROM
  TRABEVOLPREC
 WHERE
  NREVOLPRECO = @P_NREVOLPRECO AND
 (CDPRODUTO   = @P_CDPRODUTO OR @V_TCDPRODUTO = 'T');

BEGIN

-- SE NAO PASSAR O PRODUTO (NULL) SERO PROCESSADOS TODOS OS PRODUTOS
-- SENAO SERA PROCESSADO O PRODUTO PASSADO COMO PARAMETRO
  SET @V_TCDPRODUTO = '';
  IF @P_CDPRODUTO IS NULL
  BEGIN
     SET @V_TCDPRODUTO = 'T';
  END;

-- CALCULA O NUMERO DE PERIODOS PARA SELECAO DAS DATAS
  SET @V_NRPERIODOS = ROUND(CONVERT(NUMERIC, DATEDIFF(dy, @P_DTINI, @P_DTFIN)) / @P_NRPERI,0) + 1;

  OPEN C_PRODUTO;
  FETCH NEXT FROM C_PRODUTO INTO @V_CDPRODUTO
  WHILE (@@FETCH_STATUS = 0)
  BEGIN

    OPEN C_DATAS;
    FETCH NEXT FROM C_DATAS INTO @V_DATAINI, @V_DATAFIN

    WHILE (@@FETCH_STATUS = 0)
    BEGIN
      SET @V_NRCONTA = 0;
      SELECT
       @V_NRCONTA = COUNT(*)
      FROM
       TRABEVOLPREC TRAB
      WHERE
       TRAB.NREVOLPRECO  = @P_NREVOLPRECO    AND
       TRAB.CDPRODUTO    = @V_CDPRODUTO AND
       TRAB.DTINIPEREVOL = @V_DATAINI;

      IF @V_NRCONTA = 0 -- SE NAO ENCONTROU O REGISTRO DA PRODUTOS NA DATA
      BEGIN
         INSERT INTO TRABEVOLPREC
           (NREVOLPRECO,CDPRODUTO,DTINIPEREVOL,DTFINPEREVOL,
            VRPRECOEVOL, VRPERCEVOL)
           VALUES
           (@P_NREVOLPRECO,@V_CDPRODUTO,@V_DATAINI,@V_DATAFIN,
            0,0);
      END;

      FETCH NEXT FROM C_DATAS INTO
         @V_DATAINI,
         @V_DATAFIN

    END;
    CLOSE C_DATAS;
    DEALLOCATE C_DATAS;

    FETCH NEXT FROM C_PRODUTO INTO
    @V_CDPRODUTO

  END;
  CLOSE C_PRODUTO;
  DEALLOCATE C_PRODUTO;


  -- CALCULA A VARIACAO EM CADA PERIODO
  -- ((Preco do periodo atual / Preco do periodo anterior) - 1) * 100
  OPEN C_TRABEVOL;

  FETCH NEXT FROM C_TRABEVOL INTO
       @V_CDPRODUTO,
       @V_DTINIPEREVOL,
       @V_VRPRECAUX;
  SET @V_CDPRODUTOANT = @V_CDPRODUTO;
  WHILE (@@FETCH_STATUS = 0)
  BEGIN
    FETCH NEXT FROM C_TRABEVOL INTO
         @V_CDPRODUTO,
         @V_DTINIPEREVOL,
         @V_VRPRECOEVOL;
    IF @V_CDPRODUTOANT = @V_CDPRODUTO BEGIN
       IF @V_VRPRECAUX = 0 AND @V_VRPRECOEVOL = 0
       BEGIN
          SET @V_VRPERCEVOL = 0;
       END
       ELSE BEGIN IF @V_VRPRECAUX = 0
            BEGIN
              SET @V_VRPERCEVOL = 100;
            END
            ELSE BEGIN
               IF @V_VRPRECOEVOL = 0 BEGIN
                  SET @V_VRPERCEVOL = 0;
               END
               ELSE BEGIN
                  SET @V_VRPERCEVOL = ((@V_VRPRECOEVOL / @V_VRPRECAUX) - 1) * 100;
               END;
            END
       END;
       IF @V_VRPRECOEVOL = 0 BEGIN
          SET @V_VRPRECOEVOL = @V_VRPRECAUX;
       END;
    END
    ELSE BEGIN
       SET @V_VRPERCEVOL   = NULL;
       SET @V_CDPRODUTOANT = @V_CDPRODUTO;
       SET @V_VRPRECAUX    = 0;
    END;

    UPDATE TRABEVOLPREC
    SET VRPERCEVOL  = @V_VRPRECAUX,
        VRPRECOEVOL = @V_VRPRECOEVOL
    WHERE
     NREVOLPRECO  = @P_NREVOLPRECO AND
     CDPRODUTO    = @V_CDPRODUTO   AND
     DTINIPEREVOL = @V_DTINIPEREVOL;
    IF @V_VRPRECOEVOL <> 0 BEGIN
       SET @V_VRPRECAUX = @V_VRPRECOEVOL;
    END;
  END;

  DELETE TRABEVOLPREC
  WHERE
     NREVOLPRECO  = @P_NREVOLPRECO AND
    (CDPRODUTO    = @P_CDPRODUTO OR @V_TCDPRODUTO = 'T') AND
    (DTINIPEREVOL < @P_DTINI OR DTINIPEREVOL > @P_DTFIN);

  CLOSE C_TRABEVOL;
  DEALLOCATE C_TRABEVOL;
END;
GO